package fr.openium.consentium.api

import android.content.Context
import dagger.hilt.android.EntryPointAccessors
import fr.openium.consentium.api.model.PurposeChoice
import fr.openium.consentium.api.model.PurposeIdentifier
import fr.openium.consentium.api.model.PurposeStatus
import fr.openium.consentium.api.model.VendorIdentifier
import fr.openium.consentium.api.model.VendorStatus
import fr.openium.consentium.api.state.FetchConsentiumState
import fr.openium.consentium.api.state.SetConsentiumState
import fr.openium.consentium.domain.di.RepositoryEntryPoint
import fr.openium.consentium.domain.repository.ConsentiumRepository
import fr.openium.consentium.domain.repository.ConsentiumRepositoryGetResponse
import fr.openium.consentium.domain.repository.ConsentiumRepositorySetResponse
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow

class Consentium(
    context: Context,
    val applicationId: String,
) {
    private val _fetchConsentState = MutableStateFlow<FetchConsentiumState>(FetchConsentiumState.Idle)
    val fetchConsentState by lazy { _fetchConsentState.asStateFlow() }

    private val _setConsentState = MutableStateFlow<SetConsentiumState>(SetConsentiumState.Idle)
    val setConsentState by lazy { _setConsentState.asStateFlow() }

    private val consentiumRepository: ConsentiumRepository by lazy {
        val appContext = context.applicationContext
        val entryPoint = EntryPointAccessors.fromApplication(appContext, RepositoryEntryPoint::class.java)
        entryPoint.provideConsentiumRepository()
    }

    fun getConsentStatus(purpose: PurposeIdentifier): PurposeStatus {
        return when (val state = fetchConsentState.value) {
            is FetchConsentiumState.Valid -> {
                val purposeChoice = state.purposes.find { it.identifier == purpose }
                purposeChoice?.isAccepted ?: PurposeStatus.UNKNOWN
            }

            else -> PurposeStatus.UNKNOWN
        }
    }

    fun getConsentStatus(purpose: PurposeIdentifier, vendorIdentifier: VendorIdentifier): VendorStatus {
        return when (val state = fetchConsentState.value) {
            is FetchConsentiumState.Valid -> {
                val vendorChoice = state.purposes.find { it.identifier == purpose }
                    ?.vendors?.find { it.identifier == vendorIdentifier }

                when (vendorChoice?.isAccepted) {
                    true -> VendorStatus.ACCEPTED
                    false -> VendorStatus.REJECTED
                    else -> VendorStatus.UNKNOWN
                }
            }

            else -> VendorStatus.UNKNOWN
        }
    }

    suspend fun fetchConsents() {
        _fetchConsentState.value = FetchConsentiumState.Loading
        try {
            when (val consentResponse = consentiumRepository.getConsents(applicationId)) {
                ConsentiumRepositoryGetResponse.Error -> _fetchConsentState.value = FetchConsentiumState.Error

                is ConsentiumRepositoryGetResponse.GetConsentsSuccess -> {
                    val areConsentsValid = consentResponse.isValid
                    if (areConsentsValid) {
                        _fetchConsentState.value = FetchConsentiumState.Valid(purposes = consentResponse.purposes)
                    } else {
                        _fetchConsentState.value = FetchConsentiumState.Invalid
                    }
                }
            }

        } catch (e: Exception) {
            _fetchConsentState.value = FetchConsentiumState.Error
        }
    }

    suspend fun setConsents(
        consent: List<PurposeChoice>,
    ) {
        _setConsentState.value = SetConsentiumState.Loading
        try {
            when (consentiumRepository.setConsents(applicationId, consent)) {
                ConsentiumRepositorySetResponse.Error -> _setConsentState.value = SetConsentiumState.Error
                ConsentiumRepositorySetResponse.SetConsentsSuccess -> _setConsentState.value = SetConsentiumState.Success
            }

        } catch (e: Exception) {
            _setConsentState.value = SetConsentiumState.Error
        }
    }
}